package net.minecraft.src;

import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import net.minecraft.client.Minecraft;
import net.minecraft.src.forge.*;
import net.minecraft.src.forge.MessageManager.ConnectionInstance;
import net.minecraft.src.forge.packets.*;
import net.minecraft.src.ic2.api.*;
import net.minecraft.src.pwnedgod.wirelessstorage.*;
import net.minecraft.src.pwnedgod.Utils;

public class mod_PGEU extends NetworkMod
{
	static class GuiHandler implements IGuiHandler
	{
		public GuiScreen getGuiScreen(int id, EntityPlayerSP entityPlayer,
				World world, int x, int y, int z) 
		{
			TileEntity tEntity = world.getBlockTileEntity(x, y, z);
			if(tEntity != null)
			{
				Container theContainer = ((PG_TileEntityBase)tEntity).getGuiContainer(entityPlayer);
				if(id == storageGuiId) return new PG_GuiWirelessStorage((PG_ContainerStorage)theContainer);
				if(id == freqGuiId) return new PG_GuiFrequency(((PG_ContainerStorage)theContainer).mainTEntity);
				if(id == behaviorGuiId) return new PG_GuiRedstoneBehavior(((PG_ContainerStorage)theContainer).mainTEntity);
			}
			return null;
		}
		
		GuiHandler(){}
	}
	
	static class PacketHandler implements IPacketHandler
	{
		public void onPacketData(NetworkManager network, String channel, byte[] byteData) 
		{
			DataInputStream data = new DataInputStream(new ByteArrayInputStream(byteData));
			PG_PacketBase thePacket = null;
            NetClientHandler net = (NetClientHandler)network.getNetHandler();
			try
	        {
	            int packetID = data.read();
	            System.out.println(data.toString());
	            
	            switch(packetID)
	            {
	            case PG_PacketGuiSyncSend.ID:
	            	System.out.println("Processing send packet");
	            	thePacket = new PG_PacketGuiSyncSend();
	            	thePacket.readData(data);
	            	onGuiData1((PG_PacketGuiSyncSend)thePacket);
	            	break;
	            case PG_PacketGuiSyncRequest.ID:
	            	System.out.println("Processing request packet");
	            	thePacket = new PG_PacketGuiSyncRequest();
	            	thePacket.readData(data);
	            	onGuiData2((PG_PacketGuiSyncRequest)thePacket);
	            }
	        } catch(IOException e)
	        {
	            ModLoader.getLogger().severe("An error occured when trying to handle packet "+data);
	            e.printStackTrace();
	        }
		}
		
		private void onGuiData1(PG_PacketGuiSyncSend thePacket)
		{
			Block block = Block.blocksList[Utils.getWorld().getBlockId(thePacket.x, thePacket.y, thePacket.z)];
			if(Utils.getWorld().blockExists(thePacket.x, thePacket.y, thePacket.z) && block.hasTileEntity(Utils.getWorld().getBlockMetadata(thePacket.x, thePacket.y, thePacket.z)))
			{
				PG_TileEntityWirelessStorage tEntity = (PG_TileEntityWirelessStorage)Utils.getWorld().getBlockTileEntity(thePacket.x, thePacket.y, thePacket.z);
				tEntity.freq = thePacket.integers[0];
				tEntity.outputTier = thePacket.integers[1];
				tEntity.currentBehavior = thePacket.integers[2];
				tEntity.owner = thePacket.strings[0];
			}
		}
		
		private void onGuiData2(PG_PacketGuiSyncRequest thePacket)
		{
			Block block = Block.blocksList[Utils.getWorld().getBlockId(thePacket.x, thePacket.y, thePacket.z)];
			if(Utils.getWorld().blockExists(thePacket.x, thePacket.y, thePacket.z) && block.hasTileEntity(Utils.getWorld().getBlockMetadata(thePacket.x, thePacket.y, thePacket.z)))
			{
				PG_TileEntityWirelessStorage tEntity = (PG_TileEntityWirelessStorage)Utils.getWorld().getBlockTileEntity(thePacket.x, thePacket.y, thePacket.z);
				
				for(int i = 0; i < tEntity.flagFromFreq.length; i++)
				{
					tEntity.flagFromFreq[i] = thePacket.flagFromFreq[i];
				}
				
				tEntity.sourceOfStorage = thePacket.source;
				tEntity.size = thePacket.size;
				tEntity.energy = thePacket.energy;
			}
		}
		
		PacketHandler(){}
	}
	
	static class ChunkLoadingHandler implements IChunkLoadHandler
	{
		public void addActiveChunks(World world,
				Set<ChunkCoordIntPair> chunkList) 
		{
			List allStorages = PG_TileEntityWirelessStorage.storageList;
			for(int i = 0; i < allStorages.size(); i++)
			{
				PG_TileEntityWirelessStorage storageEntity = (PG_TileEntityWirelessStorage)allStorages.get(i);
				Chunk storageChunk = world.getChunkFromBlockCoords(storageEntity.xCoord, storageEntity.zCoord);
				chunkList.add(storageChunk.getChunkCoordIntPair());
			}
		}

		public boolean canUnloadChunk(Chunk chunk) 
		{
			List allStorages = PG_TileEntityWirelessStorage.storageList;
			for(int i = 0; i < allStorages.size(); i++)
			{
				PG_TileEntityWirelessStorage storageEntity = (PG_TileEntityWirelessStorage)allStorages.get(i);
				Chunk storageChunk = chunk.worldObj.getChunkFromBlockCoords(storageEntity.xCoord, storageEntity.zCoord);
				if(chunk.equals(storageChunk))
				{
					return false;
				}
			}
			return true;
		}
		
		ChunkLoadingHandler(){}
	}
	
	/*
	 * ConnectionHandler to register packets
	 */
	static class ConnectionHandler implements IConnectionHandler
	{
		IPacketHandler packetHandler;
		
		public void OnConnect(NetworkManager network) 
		{
			if(network != null && packetHandler != null)
			{
				ConnectionInstance connectionInstance = MessageManager.getInstance().getConnection(network);
				connectionInstance.registerChannel(packetHandler, "PG WS");
				if(!connectionInstance.isActiveChannel("PG WS"))
					connectionInstance.addActiveChannel("PG WS");
			}
		}

		public void OnLogin(NetworkManager network, Packet1Login login) 
		{
		}

		public void OnDisconnect(NetworkManager network, String message,
				Object[] args) 
		{
			if(network != null && packetHandler != null)
			{
				ConnectionInstance connectionInstance = MessageManager.getInstance().getConnection(network);
				if(connectionInstance.isActiveChannel("PG WS"))
					connectionInstance.removeActiveChannel("PG WS");
				connectionInstance.unregisterChannel(packetHandler, "PG WS");
				connectionInstance.unregisterHandler(packetHandler);
			}
		}
		
		ConnectionHandler()
		{
			packetHandler = new PacketHandler();
		}
	}
	
	private static boolean isMultiPlayer = false;
	private static boolean isIngame = false;
	
	public static Block storageBlock;
	
	public static Configuration config;

	public static int storageBlockId;
	public static int storageGuiId;
	public static int freqGuiId;
	public static int behaviorGuiId;
	public static int updateSpeed;
	
	public static String storageName;
	public static String excludedName;
	
	public String getPriorities()
	{
		return "after:mod_IC2";
	}
	
	public String getVersion() 
	{
		return "1.0.2";
	}

	public void load()
	{
		try
        {
			config = new Configuration(new File(Minecraft.getMinecraftDir().getPath(), "/config/IC2WirelessMFES.cfg"));
			config.load();
        }
        catch (Exception e)
        {
            System.out.println("[Wireless Storage] An error occured when trying to access configuration");
            config = null;
        }

		ModLoader.setInGUIHook(this, true, true);
		
		storageGuiId = getIntegerFor("guiIdStorage", 0, 129);
		freqGuiId = getIntegerFor("guiIdFreqScreen", 0, 130);
		behaviorGuiId = getIntegerFor("guiIdBehaviorScreen", 0, 131);
		updateSpeed = getIntegerFor("updateSpeedForPacket", 0, 250);
		
		fixStepSounds();
		
		MinecraftForge.setGuiHandler(this, new GuiHandler());
		MinecraftForge.registerConnectionHandler(new ConnectionHandler());
		MinecraftForge.registerChunkLoadHandler(new ChunkLoadingHandler());
		
		MinecraftForgeClient.preloadTexture("pwnedgod/wirelessstorage/textures/block_0.png");
	}
	
	public boolean onTickInGUI(float tick, Minecraft mc, GuiScreen gui)
	{
		if(mc.theWorld == null)
        {
            if(isIngame)
            {
                if(!isMultiPlayer)
                {
                    PG_TileEntityWirelessStorage.storageList.clear();
                    isIngame = true;
                }
                isIngame = false;
            }
        } else
        {
            isMultiPlayer = mc.theWorld.isRemote;
            isIngame = true;
        }
        return true;
	}
	
	public void modsLoaded()
	{
		super.modsLoaded();
		storageBlockId = getBlockIdFor("blockEUTeleport", 128);
		storageBlock = new PG_BlockWirelessStorage(storageBlockId);
	
		ModLoader.registerBlock(storageBlock, PG_ItemBlock.class); //The custom ItemBlock is reserved for future use.
		
		ModLoader.registerTileEntity(PG_TileEntityBase.class, "BaseTE");
		ModLoader.registerTileEntity(PG_TileEntityWirelessStorage.class, "WirelessMFESUnit");
		
		registerRecipes();
		
		Property prop = config.getOrCreateProperty("storageName", 0, "Wireless MFES Unit");
		storageName = prop.value;
		prop = config.getOrCreateProperty("excludedName", 0, "");
		excludedName = prop.value;
		
		Utils.addLocalization("wmfes.name", storageName);
		
		if(config != null) 
		{
			config.save();
		}
	}
	
	private static void fixStepSounds()
	{
		try 
		{
			Block csBlock = (Block)Class.forName("mod_CompactSolars").getField("compactSolarBlock").get(null);
			if(!csBlock.stepSound.equals(Block.soundMetalFootstep))
			{
				System.out.println("Fixing Compact Solar's step sound...");
				csBlock.setStepSound(Block.soundMetalFootstep);
			}
		}
		catch (Exception e){} 
		
		try
		{
			Block bcAutoWorkbench = (Block)Class.forName("BuildCraftFactory").getField("autoWorkbenchBlock").get(null);
			if(!bcAutoWorkbench.stepSound.equals(Block.soundWoodFootstep))
			{
				System.out.println("Fixing Auto Crafting table's step sound...");
				bcAutoWorkbench.setStepSound(Block.soundWoodFootstep);
			}
		}
		catch (Exception e){} 
	}
	
	private static void registerRecipes()
	{
		Ic2Recipes.addCraftingRecipe(new ItemStack(storageBlock), new Object[]
				{
					"OUL", "MTM", "P#P", 
					Character.valueOf('T'), Items.getItem("teleporter"),
					Character.valueOf('M'), Items.getItem("mfsUnit"),
					Character.valueOf('#'), Items.getItem("matter"),
					Character.valueOf('O'), Items.getItem("lvTransformer"),
					Character.valueOf('U'), Items.getItem("mvTransformer"),
					Character.valueOf('L'), Items.getItem("hvTransformer"),
					Character.valueOf('P'), Item.enderPearl
				});
		Ic2Recipes.addCraftingRecipe(new ItemStack(Items.getItem("doubleInsulatedGoldCableItem").getItem(), 4), new Object[]
                {
                    "RRR", "RGR", "RRR", 
                    Character.valueOf('R'), Items.getItem("rubber"),
                    Character.valueOf('G'), Item.ingotGold
                });
		Ic2Recipes.addCraftingRecipe(new ItemStack(Items.getItem("doubleInsulatedIronCableItem").getItem(), 4), new Object[]
                {
                    "RRR", "RIR", "RRR", 
                    Character.valueOf('R'), Items.getItem("rubber"),
                    Character.valueOf('I'), Items.getItem("refinedIronIngot")
                });
	}
	
	protected static int getBlockIdFor(String name, int defaultValue)
    {
        if (config == null)
        {
            return defaultValue;
        } else
        {
            try
            {
                return Integer.valueOf(config.getOrCreateBlockIdProperty(name, defaultValue).value).intValue();
            }
            catch (Exception e)
            {
                System.out.println("[Wireless Storage] An error occured while trying to access config, wasn't loaded properly!");
                return defaultValue;
            }
        }
    }
	
	protected static int getIntegerFor(String name, int kind, int defaultValue)
    {
        if (config == null)
        {
            return defaultValue;
        } else
        {
            try
            {
                return Integer.valueOf(config.getOrCreateIntProperty(name, kind, defaultValue).value).intValue();
            }
            catch (Exception e)
            {
                System.out.println("[Wireless Storage] An error occured while trying to access config, wasn't loaded properly!");
                return defaultValue;
            }
        }
    }
	
	public static void openGuiFor(World world, PG_TileEntityBase tEntity, EntityPlayer entityPlayer)
	{
		switch(world.getBlockMetadata(tEntity.xCoord, tEntity.yCoord, tEntity.zCoord))
		{
		case 0: ((EntityPlayerSP)entityPlayer).openGui(Utils.getModInstance(mod_PGEU.class), storageGuiId, world, tEntity.xCoord, tEntity.yCoord, tEntity.zCoord);
		}
	}
	
	public boolean clientSideRequired(){return true;}
	public boolean serverSideRequired(){return true;}
}
