package net.minecraft.src.pwnedgod;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import net.minecraft.client.Minecraft;
import net.minecraft.src.*;
import net.minecraft.src.forge.*;

public final class Utils 
{
	public static Minecraft mc = ModLoader.getMinecraftInstance();
	
	/**
	 * Gets the world's save directory
	 * <p> credit goes to chickenbones for this code even though this went unused
	 * @param world the World's instance
	 * @return the World's save directory
	 * @author chickenbones
	 */
	public static File getWorldSaveDir(World world)
	{
		File savedir = getWorldSaveDirLocation(world);
		if(savedir == null)
		{
			return null;
		} else if(savedir.getName().contains("DIM"))
		{
			return savedir.getParentFile();
		} else
		{
			return savedir;
		}
	}
	
	/**
	 * Gets the world's dimension's save directory
	 * <p> credit goes to chickenbones for this code even though this went unused
	 * @param world the World's instance
	 * @return the World's dimension save directory
	 * @author chickenbones
	 */
	public static File getWorldSaveLocation(World world, int dimension)
	{
		File basesave = getWorldSaveDir(world); 
		if(dimension != 0)
		{
			return new File(basesave,"DIM" + dimension);
		} else
		{
			return basesave;
		}
	}
	
	private static File getWorldSaveDirLocation(World world)
	{
		try
		{
			ISaveHandler worldsaver = world.getSaveHandler();
			IChunkLoader loader = worldsaver.getChunkLoader(world.worldProvider);
			if(loader instanceof AnvilChunkLoader)
			{
				return (File)ModLoader.getPrivateValue(AnvilChunkLoader.class, loader, 3);
			}
			return null;
		} catch(Exception e)
		{
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	 * Creates a explosion at the specified Coordinates
	 * @param world the World instance
	 * @param x x Position
	 * @param y y Position
	 * @param z z Position
	 */
	public static void explodeMachineAt(World world, int x, int y, int z)
	{
		try 
		{
			Class.forName("mod_IC2").getMethod("explodeMachineAt", World.class, Integer.TYPE, Integer.TYPE, Integer.TYPE).invoke(null, world, x, y, z);
		} catch (Exception e)
		{
			System.out.println("An error occured when trying to explode machine. Using default explosion...");
			world.setBlockWithNotify(x, y, z, 0);
			world.createExplosion(null, (double)x, (double)y, (double)z, 2.5F);
		}
	}
	
	/**
	 * Creates a icon button
	 * <p>This uses IC2's Icon Button. If IC2 does not exist it will instead return a blank button
	 * @param id Button id
	 * @param x x Position
	 * @param y y Position
	 * @param w width
	 * @param h height
	 * @param texture texture file
	 * @param textureX texture's x Position
	 * @param textureY texture's y Position
	 * @return a button with the specified icon. If IC2 does not exist it will instead be a blank button
	 */
	public static GuiButton iconButton(int id, int x, int y, int w, int h, String texture, int textureX, int textureY)
	{
		try 
		{
			return (GuiButton)Class.forName("ic2.platform.GuiIconButton").getConstructor(Integer.TYPE, Integer.TYPE, Integer.TYPE, Integer.TYPE, Integer.TYPE, String.class, Integer.TYPE, Integer.TYPE).newInstance(id, x, y, w, h, texture, textureX, textureY);
		} catch (Exception e)
		{
			return new GuiButton(id, x, y, w, h, "");
		}
	}
	
	/**
	 * Gets the player's instance
	 * @return the player's instance 
	 */
	public static EntityPlayerSP getPlayerInstance()
	{
		return mc.thePlayer;
	}
	
	/**
	 * Gets the world's instance
	 * @return the world's instance 
	 */
	public static World getWorld()
	{
		return mc.theWorld;
	}
	
	/**
	 * Gets the name from the TileEntity
	 * @param tEntity the TileEntity's class
	 * @return the TileEntity's registered name
	 */
	public static String getTileEntityName(Class tEntity)
	{
		String name = "";
		try
		{
			Map map = (Map)ModLoader.getPrivateValue(net.minecraft.src.TileEntity.class, new TileEntity(), 1);
			name = (String)map.get(tEntity);
		} catch(Exception e){}
		return name;
	}
	
	/**
	 * Gets the TileEntity class from a specified name
	 * @param tEntityRegisteredName the name from the RegisterTileEntity
	 * @return the TileEntity's class
	 */
	public static Class getTileEntityClassFromName(String tEntityRegisteredName)
	{
		Class cls = null;
		try
		{
			Map map = (Map)ModLoader.getPrivateValue(net.minecraft.src.TileEntity.class, new TileEntity(), 0);
			cls = (Class)map.get(tEntityRegisteredName);
		} catch(Exception e){}
		return cls;
	}

	/**
	 * A modified version of {@link ModLoaderMp#getModInstance}
	 * @param modClass the mod's class
	 * @return the instance of the mod
	 */
	public static BaseMod getModInstance(Class modClass)
	{
		BaseMod theMod = null;
		for (int i = 0; i < ModLoader.getLoadedMods().size(); i++)
		{
			theMod = (BaseMod)ModLoader.getLoadedMods().get(i);
			if (modClass.isInstance(theMod))
			{
				break;
			}
        }
        return theMod;
    }
	
	/**
	 * Gets all field from the specified type.
	 * @param targetClass The target class
	 * @param type the type to get
	 * @param instance instance of the object
	 * @return a list of the specified fields
	 */
	public static List getAllSpecifiedTypeOfField(Class targetClass, Class<?> type, Object instance) 
	{
		List temp = new LinkedList();
		Field[] allFields = targetClass.getDeclaredFields();
		for(int i = 0; i < allFields.length; i++)
		{
			try
			{
				if(allFields[i].getType().equals(type) || allFields[i].getType() == type)
				{
					allFields[i].setAccessible(true);
					temp.add(allFields[i].get(instance));
				}
			} catch(Exception e){}
		}
		return temp;
	}
	
	/**
	 * Sends a packet.
	 * @param packet the packet that needs to be send
	 */
	public static void sendPacket(Packet packet)
	{
		mc.getSendQueue().addToSendQueue(packet);
	}
	
	/**
	 * Gets whether or not the Minecraft simulation is on client
	 * @return whether or not it is the client.
	 */
	public static boolean isOnClient()
	{
		return true;
	}
	
	/**
	 * Adds a localization. This will be only useful if it's on a server.
	 * @param key
	 * @param name
	 */
	public static void addLocalization(String key, String name)
	{
		ModLoader.addLocalization(key, name);
	}
	
	/**
	 * Gets the simulation type. Useful for TileEntities
	 * @return true if the world is SSP and false if the World is SMP
	 */
	public static boolean isSimulating()
	{
		return !getWorld().isRemote;
	}
	
	/**
	 * Gets whether or not the Minecraft simulation is on server
	 * @return true if the world is SMP and false if the World is SSP
	 */
	public static boolean isOnServer()
	{
		return getWorld().isRemote;
	}
}
