package net.minecraft.src.pwnedgod.wirelessstorage;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;
import net.minecraft.src.*;
import net.minecraft.src.forge.ITextureProvider;
import net.minecraft.src.ic2.api.Items;
import net.minecraft.src.pwnedgod.Utils;

public class PG_BlockWirelessStorage extends BlockContainer implements ITextureProvider
{
	public static final int[][] sideAndFacingToSpriteOffset = {
		{3, 2, 0, 0, 0, 0}, 
		{2, 3, 1, 1, 1, 1}, 
		{1, 1, 3, 2, 5, 4}, 
		{0, 0, 2, 3, 4, 5}, 
		{4, 5, 4, 5, 3, 2}, 
		{5, 4, 5, 4, 2, 3}};

	
	public PG_BlockWirelessStorage(int id) 
	{
		super(id, Material.iron);
		setStepSound(soundMetalFootstep);
		setHardness(2.0F);
	}

	public boolean isPoweringTo(IBlockAccess iblockaccess, int x, int y, int z, int side) 
	{
		TileEntity tEntity = iblockaccess.getBlockTileEntity(x, y, z);
		if(tEntity instanceof PG_TileEntityWirelessStorage) 
		{
			PG_TileEntityWirelessStorage electricBlock = (PG_TileEntityWirelessStorage)tEntity;
			return electricBlock.isEmittingRedstone();
		} else 
		{
			return false;
		}
	}
	
	public boolean isIndirectlyPoweringTo(World world, int x, int y, int z, int side) 
	{
		return isPoweringTo(world, x, y, z, side);
	}
	
	public boolean canProvidePower()
	{
		return true;
	}
	
	public int getBlockTexture(IBlockAccess iblockaccess, int x, int y, int z, int side)
	{
		TileEntity tEntity = iblockaccess.getBlockTileEntity(x, y, z);
		short facing = tEntity instanceof PG_TileEntityBase ? ((PG_TileEntityBase)tEntity).getFacing() : 0 ;
		int meta = iblockaccess.getBlockMetadata(x, y, z);
		return isActive(iblockaccess, x, y, z) ? meta + (sideAndFacingToSpriteOffset[side][facing] + 6) * 16 : meta + sideAndFacingToSpriteOffset[side][facing] * 16;
	}
	
	public int getBlockTextureFromSideAndMetadata(int side, int meta)
	{
		return meta + sideAndFacingToSpriteOffset[side][3] * 16;
	}
	
	public String getTextureFile()
    {
        return "/pwnedgod/wirelessstorage/textures/block_0.png";
    }

	public TileEntity getBlockEntity() 
	{
		return null;
	}
	
	public TileEntity getBlockEntity(int meta) 
	{
		return new PG_TileEntityWirelessStorage();
	}
	
	public ArrayList getBlockDropped(World world, int x, int y, int z, int meta, int fortune)
    {
        ArrayList array = super.getBlockDropped(world, x, y, z, meta, fortune);
        TileEntity tEntity = world.getBlockTileEntity(x, y, z);
        if(tEntity != null)
        {
        	if (tEntity instanceof IInventory)
        	{
            	IInventory theInventory = (IInventory)tEntity;

            	for (int i = 0; i < theInventory.getSizeInventory(); i++)
            	{
                	ItemStack is = theInventory.getStackInSlot(i);
                	if (is != null)
                	{
                    	array.add(is);
                    	theInventory.setInventorySlotContents(i, (ItemStack)null);
                	}
            	}
        	}
        }
        return array;
    }
	
	public void onBlockRemoval(World world, int x, int y, int z)
    {
		super.onBlockRemoval(world, x, y, z);
        boolean flag = true;
        TileEntity tEntity = world.getBlockTileEntity(x, y, z);
        if(tEntity != null && tEntity instanceof PG_TileEntityWirelessStorage)
        {
        	((PG_TileEntityWirelessStorage)tEntity).resetAmount();
        }
        for (Iterator it = getBlockDropped(world, x, y, z, world.getBlockMetadata(x, y, z), 0).iterator(); it.hasNext(); flag = false)
        {
            ItemStack is = (ItemStack)it.next();
            if (!flag)
            {
                if (is == null)
                {
                    return;
                }

                double var8 = 0.7D;
                double var10 = (double)world.rand.nextFloat() * var8 + (1.0D - var8) * 0.5D;
                double var12 = (double)world.rand.nextFloat() * var8 + (1.0D - var8) * 0.5D;
                double var14 = (double)world.rand.nextFloat() * var8 + (1.0D - var8) * 0.5D;
                EntityItem itemEntity = new EntityItem(world, (double)x + var10, (double)y + var12, (double)z + var14, is);
                itemEntity.delayBeforeCanPickup = 10;
                world.spawnEntityInWorld(itemEntity);
                return;
            }
        }
    }
	
	public void onBlockPlacedBy(World world, int x, int y, int z, EntityLiving entityLiving) 
	{
		super.onBlockPlacedBy(world, x, y, z, entityLiving);
		if(Utils.isSimulating()) 
		{
			PG_TileEntityBase tEntity = (PG_TileEntityBase)world.getBlockTileEntity(x, y, z);
			if(entityLiving == null) 
			{
				tEntity.setFacing((short)1);
			} else 
			{
				int yaw = MathHelper.floor_double((double)(entityLiving.rotationYaw * 4.0F / 360.0F) + 0.5D) & 3;
				int pitch = Math.round(entityLiving.rotationPitch);
				if(pitch >= 65) 
				{
					tEntity.setFacing((short)1);
				} else if(pitch <= -65) 
				{
					tEntity.setFacing((short)0);
				} else 
				{
					switch(yaw) 
					{
					case 0:tEntity.setFacing((short)2);break;
					case 1:tEntity.setFacing((short)5);break;
					case 2:tEntity.setFacing((short)3);break;
					case 3:tEntity.setFacing((short)4);
					}
	            }
				if(entityLiving instanceof EntityPlayer)
				{
					((PG_TileEntityWirelessStorage)tEntity).setOwnerFrom((EntityPlayer)entityLiving);
				}
			}
		}
	}
	
	public static boolean isActive(IBlockAccess iblockaccess, int i, int j, int k) 
	{
		TileEntity te = iblockaccess.getBlockTileEntity(i, j, k);
		return te instanceof PG_TileEntityBase ? ((PG_TileEntityBase)te).getActive() : false;
	}
	
	public boolean blockActivated(World world, int x, int y, int z, EntityPlayer entityPlayer)
    {
        if (entityPlayer.isSneaking())
        {
        	return false;
        } else
        {
            if(Utils.isSimulating())
            {
            	PG_TileEntityBase tEntity = (PG_TileEntityBase)world.getBlockTileEntity(x, y, z);
            	if(tEntity != null)
            	{
            		if(tEntity instanceof PG_TileEntityWirelessStorage ? ((PG_TileEntityWirelessStorage)tEntity).isTheCorrectOwner(entityPlayer) : true)
            		{
            			mod_PGEU.openGuiFor(world, tEntity, entityPlayer);
            			return true;
            		}
            	}
            }
            return false;
        }
    }
	
	public int idDropped(int meta, Random random, int j)
	{
		return Items.getItem("advancedMachine").itemID;
	}
	
	public int damageDropped(int meta)
	{
		return Items.getItem("advancedMachine").getItemDamage();
	}
}
