package net.minecraft.src.pwnedgod.wirelessstorage;

import java.util.List;
import java.util.Vector;
import net.minecraft.src.*;
import net.minecraft.src.ic2.api.*;
import net.minecraft.src.pwnedgod.Utils;

public class PG_TileEntityBase extends TileEntity
implements INetworkUpdateListener, INetworkDataProvider, IWrenchable
{
	public PG_TileEntityBase()
    {
		created = false;
		active = false;
		facing = 0;
        prevActive = false;
        prevFacing = 0;
    }
	
	public void onCreated()
    {
        if(!Utils.isSimulating())
        {
            NetworkHelper.requestInitialData(this);
        }
    }
	
	public void updateEntity()
    {
        if (!created)
        {
        	onCreated();
        	created = true;
        }
    }
	
	public boolean getActive()
    {
        return active;
    }
	
	public Container getGuiContainer(EntityPlayer entityPlayer)
	{
		return null;
	}
	
	public void setActive(boolean flag)
    {
		active = flag;
        if(prevActive != active)
        {
        	NetworkHelper.updateTileEntityField(this, "active");
        }
        prevActive = flag;
    }
    
    public void setActiveWithoutNotify(boolean flag)
    {
    	active = flag;
        prevActive = flag;
    }
	
	public void readFromNBT(NBTTagCompound nbttagcompound)
    {
        super.readFromNBT(nbttagcompound);
        prevFacing = facing = nbttagcompound.getShort("facing");
        setActive(nbttagcompound.getBoolean("active"));
    }

    public void writeToNBT(NBTTagCompound nbttagcompound)
    {
        super.writeToNBT(nbttagcompound);
        nbttagcompound.setShort("facing", facing);
        nbttagcompound.setBoolean("active", active);
    }
	
    public void onNetworkUpdate(String field)
    {
        if(field.equals("active") && prevActive != active || field.equals("facing") && prevFacing != facing)
        {
            worldObj.markBlockNeedsUpdate(xCoord, yCoord, zCoord);
            prevActive = active;
            prevFacing = facing;
        }
    }
    
	public List getNetworkedFields()
	{
		List ret = new Vector(2);
    	ret.add("active");
    	ret.add("facing");
    	return ret;
	}
	
	protected boolean created;
    private boolean active;
    private short facing;
    public boolean prevActive;
    public short prevFacing;

	public boolean wrenchCanSetFacing(EntityPlayer entityPlayer, int side) 
	{
		return false;
	}

	public short getFacing() 
	{
		return facing;
	}

	public void setFacing(short facing) 
	{
		this.facing = facing;
        if(prevFacing != facing)
            NetworkHelper.updateTileEntityField(this, "facing");
        prevFacing = facing;
	}

	public boolean wrenchCanRemove(EntityPlayer entityPlayer) 
	{
		return true;
	}

	public float getWrenchDropRate() 
	{
		return 1.0F;
	}
}
