package net.minecraft.src.pwnedgod.wirelessstorage;

import java.io.*;
import java.util.*;
import net.minecraft.src.Container;
import net.minecraft.src.EntityPlayer;
import net.minecraft.src.Item;
import net.minecraft.src.ItemStack;
import net.minecraft.src.NBTBase;
import net.minecraft.src.NBTTagCompound;
import net.minecraft.src.NBTTagList;
import net.minecraft.src.Packet;
import net.minecraft.src.TileEntity;
import net.minecraft.src.mod_PGEU;
import net.minecraft.src.forge.*;
import net.minecraft.src.ic2.api.*;
import net.minecraft.src.pwnedgod.Utils;

public class PG_TileEntityWirelessStorage extends PG_TileEntityBase
implements IEnergySink, IEnergySource, ISidedInventory
{
	public PG_TileEntityWirelessStorage()
	{
		//type: Boolean
		addedToEnergyNet = false;
		ownerIsSet = false;
		sourceOfStorage = false;
		resetted = false;
		
		//type: Container
		emptyContainer = new PG_ContainerEmpty();
		
		//type: Integer
		maxStorage = 0x1312d00;
		tier = 3;
		freq = 0;
		outputTier = 2;
		currentBehavior = 0;
		prevFreq = 0;
		
		//type: Integer from array
		output = aOutput[outputTier];
		energy = energyOnFreq[freq];
		
		//type: Array Integer
		inputForItemToCharge = new int[]{0};
		inputForItemToDischarge = new int[]{1};
		
		//type: Array ItemStack
		inventory = new ItemStack[2];
		
		//type: Array Boolean
		flagFromFreq = new boolean[energyOnFreq.length];
		
		//type: String
		owner = "";
	}
	
	public void updateEntity()
	{
		super.updateEntity();
		if(!storageList.contains(this))
		{
			storageList.add(this);
		}
        if(Utils.isSimulating())
        {
        	if(!sourceOfStorage)
        	{
        		List temp = new LinkedList();
            	for(int i = 0; i < getSharingStorages().size(); i++)
        		{
        			if(((PG_TileEntityWirelessStorage)getSharingStorages().get(i)).sourceOfStorage)
        				temp.add((PG_TileEntityWirelessStorage)getSharingStorages().get(i));
        		}
            	if(temp.size() < 1 && energy >= 1)
    				sourceOfStorage = true;
            	temp.clear();
        	}
        	
            if(!addedToEnergyNet)
            {
                EnergyNet.getForWorld(worldObj).addTileEntity(this);
                addedToEnergyNet = true;
            }
            boolean needsInvUpdate = false;
            if(energy > 0 && inventory[0] != null && (Item.itemsList[inventory[0].itemID] instanceof IElectricItem))
            {
                int sent = ElectricItem.charge(inventory[0], energy, tier, false, false);
                energyOnFreq[freq] -= sent;
                needsInvUpdate = sent > 0;
            }
            if(demandsEnergy() && inventory[1] != null)
            {
                if(Item.itemsList[inventory[1].itemID] instanceof IElectricItem)
                {
                    IElectricItem item = (IElectricItem)Item.itemsList[inventory[1].itemID];
                    if(item.canProvideEnergy())
                    {
                        int gain = ElectricItem.discharge(inventory[1], maxStorage - energy, tier, false, false);
                        energyOnFreq[freq] += gain;
                        needsInvUpdate = gain > 0;
                    }
                } else
                {
                    int id = inventory[1].itemID;
                    int gain = 0;
                    if(id == Item.redstone.shiftedIndex)
                        gain = 500;
                    if(id == Items.getItem("suBattery").itemID)
                        gain = 1000;
                    if(gain > 0 && gain <= maxStorage - energy)
                    {
                    	inventory[1].stackSize--;
                        if(inventory[1].stackSize <= 0)
                        	inventory[1] = null;
                        energyOnFreq[freq] += gain;
                    }
                }
            }
            if(prevFreq == freq)
            {
            	energy = energyOnFreq[freq];
            } else
            {
            	prevFreq = freq;
            }
            boolean isFull = false;
            if(energy >= maxStorage) isFull = true;
            
            setActive(isFull);
            worldObj.notifyBlocksOfNeighborChange(xCoord, yCoord, zCoord, worldObj.getBlockId(xCoord, yCoord, zCoord));
            
            if(energy >= output && (currentBehavior != 4 || !worldObj.isBlockIndirectlyGettingPowered(xCoord, yCoord, zCoord)) && (currentBehavior != 5 || !worldObj.isBlockIndirectlyGettingPowered(xCoord, yCoord, zCoord) || energy >= maxStorage))
            {
            	int sent = output - EnergyNet.getForWorld(worldObj).emitEnergyFrom(this, output);
            	energyOnFreq[freq] -= sent;
            }
            
            if(needsInvUpdate) onInventoryChanged();
            
            if(inventory[0] != null)
            	charging = ElectricItem.charge(inventory[0], energy, tier, false, true) > 0;
            else
            	charging = false;
            
           
            if(inventory[1] != null && Item.itemsList[inventory[1].itemID] instanceof IElectricItem)
            {
            	if(((IElectricItem)Item.itemsList[inventory[1].itemID]).canProvideEnergy())
            		discharging = ElectricItem.discharge(inventory[1], maxStorage - energy, tier, false, true) > 0;
            	else
            		discharging = false;
            } else
            {
            	discharging = false;
            }
            output = aOutput[outputTier];
        }
        switch(currentBehavior)
		{
		case 0: hPosForIcon = 0; currentBehaviorName = "Nothing"; break;
		case 1: hPosForIcon = 16; currentBehaviorName = "Emit if full"; break;
		case 2: hPosForIcon = 32; currentBehaviorName = "Emit if partially filled"; break;
		case 3: hPosForIcon = 48; currentBehaviorName = "Emit if completely empty"; break;
		case 4: hPosForIcon = 64; currentBehaviorName = "Emit if not enough power"; break;
		case 5: hPosForIcon = 80; currentBehaviorName = "Do not output energy"; break;
		case 6: hPosForIcon = 96; currentBehaviorName = "Do not output energy unless full";
		}
        removeOld();
        if(Utils.isOnServer() && Utils.isOnClient())
        {
        	if(counts == mod_PGEU.updateSpeed)
        	{
        		Utils.sendPacket(requestGuiData());
        		counts = 0;
        	} else
        	{
        		counts++;
        	}
        }
	}
	
	public void setOwnerFrom(EntityPlayer entityPlayer)
	{
		if(!ownerIsSet)
		{
			owner = entityPlayer.username;
			ownerIsSet = true;
		}
	}
	
	public void invalidate()
	{
		if(Utils.isSimulating())
        {
			if(addedToEnergyNet)
			{
				EnergyNet.getForWorld(worldObj).removeTileEntity(this);
				addedToEnergyNet = false;
			}
			owner = "";
        }
		resetAmount();
		super.invalidate();
	}
	
	public void resetAmount()
	{
		if(storageList.contains(this))
		{
			storageList.remove(this);
		}
		if(Utils.isSimulating() && !resetted && !isInvalid())
		{
			if(sourceOfStorage)
			{
				if(getSharingStorages().size() < 1)
				{
					energyOnFreq[freq] = 0;
				} else
				{
					List connectedStorage = getSharingStorages();
					int target = 0;
					for(int i = 0; i > connectedStorage.size(); i++)
					{
						if((PG_TileEntityWirelessStorage)connectedStorage.get(i) != null && !((PG_TileEntityWirelessStorage)connectedStorage.get(i)).sourceOfStorage)
						{
							target = i;
							break;
						}
					}
					((PG_TileEntityWirelessStorage)connectedStorage.get(target)).sourceOfStorage = true;
				}
			}
			if(!owner.equals("")) owner = "";
			resetted = true;
		}
	}
	
	public boolean acceptsEnergyFrom(TileEntity emitter, Direction direction) 
	{
		return !facingMatchesDirection(direction) && (emitter instanceof PG_TileEntityWirelessStorage ? ((PG_TileEntityWirelessStorage)emitter).freq != freq || !((PG_TileEntityWirelessStorage)emitter).owner.equalsIgnoreCase(owner) : true);
	}

	public boolean isAddedToEnergyNet()
	{
		return addedToEnergyNet;
	}
	
	public boolean emitsEnergyTo(TileEntity receiver, Direction direction) 
	{
		return facingMatchesDirection(direction) && (receiver instanceof PG_TileEntityWirelessStorage ? ((PG_TileEntityWirelessStorage)receiver).freq != freq || !((PG_TileEntityWirelessStorage)receiver).owner.equalsIgnoreCase(owner) : true);
	}
	
	public boolean wrenchCanSetFacing(EntityPlayer entityPlayer, int side)
    {
        return getFacing() != side;
    }

    public void setFacing(short facing)
    {
        if(addedToEnergyNet)
            EnergyNet.getForWorld(worldObj).removeTileEntity(this);
        addedToEnergyNet = false;
        super.setFacing(facing);
        EnergyNet.getForWorld(worldObj).addTileEntity(this);
        addedToEnergyNet = true;
    }
    
	public boolean facingMatchesDirection(Direction direction)
    {
        return direction.toSideValue() == getFacing();
    }
	
	public int getMaxEnergyOutput() 
	{
		return output;
	}

	public boolean demandsEnergy() 
	{
		return energy < maxStorage;
	}

	public int injectEnergy(Direction directionFrom, int amount) 
	{
        int need = amount;
        if(amount > 2048)
        {
            Utils.explodeMachineAt(worldObj, xCoord, yCoord, zCoord);
            return 0;
        } else
        {
        	if(energy + amount >= maxStorage + output)
        		need = (maxStorage + output) - energy - 1;
        	
        	energyOnFreq[freq] += need;
        	return amount - need;
        }
	}

	public int getSizeInventory() 
	{
		return inventory.length;
	}
	
	public ItemStack getStackInSlotOnClosing(int i) 
	{
		if (inventory[i] != null)
        {
            ItemStack is = inventory[i];
            inventory[i] = null;
            return is;
        } else
        {
            return null;
        }
	}

	public ItemStack getStackInSlot(int i) 
	{
		return inventory[i];
	}
	
	public void readFromNBT(NBTTagCompound nbttagcompound)
    {
        super.readFromNBT(nbttagcompound);
        NBTTagList nbttaglist = nbttagcompound.getTagList("Items");
        inventory = new ItemStack[getSizeInventory()];
        
        for(int i = 0; i < nbttaglist.tagCount(); i++)
        {
            NBTTagCompound nbttagcompound1 = (NBTTagCompound)nbttaglist.tagAt(i);
            byte byte0 = nbttagcompound1.getByte("Slot");
            if(byte0 >= 0 && byte0 < inventory.length)
            	inventory[byte0] = ItemStack.loadItemStackFromNBT(nbttagcompound1);
        }
        
        setActiveWithoutNotify(nbttagcompound.getBoolean("active"));
        sourceOfStorage = nbttagcompound.getBoolean("source");
        freq = nbttagcompound.getInteger("frequency");
        outputTier = nbttagcompound.getInteger("outputtier");
        currentBehavior = nbttagcompound.getInteger("behavior");
        
        if(sourceOfStorage)
        {
        	if(maxStorage > Integer.MAX_VALUE)
            {
            	energy *= 10;
            }
        	energy = nbttagcompound.getInteger("energy");
        	energyOnFreq[freq] = energy;
        }
    }

    public void writeToNBT(NBTTagCompound nbttagcompound)
    {
        super.writeToNBT(nbttagcompound);
        NBTTagList nbttaglist = new NBTTagList();
        
        for(int i = 0; i < inventory.length; i++)
        {
            if(inventory[i] != null)
            {
            	NBTTagCompound nbttagcompound1 = new NBTTagCompound();
                nbttagcompound1.setByte("Slot", (byte)i);
                inventory[i].writeToNBT(nbttagcompound1);
                nbttaglist.appendTag(nbttagcompound1);
            }
        }
        nbttagcompound.setTag("Items", nbttaglist);
        
        nbttagcompound.setBoolean("active", getActive());
        nbttagcompound.setBoolean("source", sourceOfStorage);
        nbttagcompound.setInteger("frequency", freq);
        nbttagcompound.setInteger("outputtier", outputTier);
        nbttagcompound.setInteger("behavior", currentBehavior);
       
        if(sourceOfStorage)
        {
        	int write = energy;
        	if(maxStorage > Integer.MAX_VALUE) 
        	{
        		write /= 10;
        	}
        	nbttagcompound.setInteger("energy", write);
        }
    }

	public ItemStack decrStackSize(int i, int j)
	{
		if(inventory[i] != null)
        {
            if(inventory[i].stackSize <= j)
            {
                ItemStack itemstack = inventory[i];
                inventory[i] = null;
                return itemstack;
            }
            ItemStack itemstack1 = inventory[i].splitStack(j);
            if(inventory[i].stackSize == 0)
            	inventory[i] = null;
            return itemstack1;
        } else
        {
            return null;
        }
	}

	public void setInventorySlotContents(int i, ItemStack itemstack)
	{
		inventory[i] = itemstack;
        if(itemstack != null && itemstack.stackSize > getInventoryStackLimit())
            itemstack.stackSize = getInventoryStackLimit();
	}

	public String getInvName() 
	{
		return "Wireless MFES Unit";
	}

	public int getInventoryStackLimit()
	{
		return 64;
	}

	public boolean isUseableByPlayer(EntityPlayer entityplayer) 
	{
		return worldObj.getBlockTileEntity(xCoord, yCoord, zCoord) != this ? false : entityplayer.getDistance((double)xCoord + 0.5D, (double)yCoord + 0.5D, (double)zCoord + 0.5D) <= 64D;
	}

	public void openChest(){}

	public void closeChest(){}

	public int getStartInventorySide(int side) 
	{
		switch(side)
        {
        case 0:return 1;
        case 1:
        default:return 0;
        }
	}

	public int getSizeInventorySide(int side) 
	{
		return 1;
	}
	
	public boolean isDischarging()
	{
		return discharging;
	}
	
	public boolean isCharging()
	{
		return charging;
	}
	
	public int gaugeEnergyScaled(int i)
    {
        return (energy * i) / maxStorage;
    }
	
	public boolean isTheCorrectOwner(EntityPlayer entityPlayer)
	{
		return entityPlayer.username.equalsIgnoreCase(owner) || owner.equalsIgnoreCase("") || (owner.startsWith("player") && !owner.equalsIgnoreCase(mod_PGEU.excludedName));
	}
	
	public Container getGuiContainer(EntityPlayer entityPlayer)
	{
		return new PG_ContainerStorage(entityPlayer.inventory, this);
	}
	
	public boolean isEmittingRedstone() 
	{
		switch(currentBehavior) 
		{
		case 1:return energy >= maxStorage;
		case 2:return energy > output && energy < maxStorage;
		case 3:return energy == 0;
		case 4:return energy < output;
		default:return false;
		}
	}
	
	public List removeOld()
	{
		List toRemove = new LinkedList();
		for(int i = 0; i < storageList.size(); i++)
		{
			PG_TileEntityWirelessStorage tEntity = (PG_TileEntityWirelessStorage)storageList.get(i);
			if((tEntity == null || !(tEntity instanceof PG_TileEntityWirelessStorage)) && worldObj.getWorldInfo().getDimension() == tEntity.worldObj.getWorldInfo().getDimension())
				toRemove.add(storageList.get(i));
		}
		storageList.removeAll(toRemove);
		return toRemove;
	}
	
	public List getSharingStorages()
	{
		List temp = new LinkedList();
		removeOld();
		for(int i = 0; i < storageList.size(); i++)
		{
			PG_TileEntityWirelessStorage tEntity = (PG_TileEntityWirelessStorage)storageList.get(i);
			if((tEntity.owner.equalsIgnoreCase(owner) || 
					Utils.isOnClient()) && 
					tEntity.freq == freq && 
					(xCoord != tEntity.xCoord || 
					yCoord != tEntity.yCoord || 
					zCoord != tEntity.zCoord ||
					worldObj.getWorldInfo().getDimension() != tEntity.worldObj.getWorldInfo().getDimension()))
				temp.add(storageList.get(i));
		}
		return temp;
	}
	
	public List getStoragesOnSpecifiedFreq(int freq)
	{
		List temp = new LinkedList();
		removeOld();
		for(int i = 0; i < storageList.size(); i++)
		{
			PG_TileEntityWirelessStorage tEntity = (PG_TileEntityWirelessStorage)storageList.get(i);
			if((tEntity.owner.equalsIgnoreCase(owner) || 
					Utils.isOnClient()) && 
					tEntity.freq == freq && 
					(xCoord != tEntity.xCoord || 
					yCoord != tEntity.yCoord || 
					zCoord != tEntity.zCoord ||
					worldObj.getWorldInfo().getDimension() != tEntity.worldObj.getWorldInfo().getDimension()))
				temp.add(storageList.get(i));
		}
		return temp;
	}
	
	public boolean isThereSourceOn(int freq)
	{
		List temp = new LinkedList();
		removeOld();
		for(int i = 0; i <getStoragesOnSpecifiedFreq(freq).size(); i++)
		{
			if(((PG_TileEntityWirelessStorage)getStoragesOnSpecifiedFreq(freq).get(i)).sourceOfStorage &&
					(((PG_TileEntityWirelessStorage)getStoragesOnSpecifiedFreq(freq).get(i)).xCoord != xCoord
					|| ((PG_TileEntityWirelessStorage)getStoragesOnSpecifiedFreq(freq).get(i)).yCoord != yCoord
					|| ((PG_TileEntityWirelessStorage)getStoragesOnSpecifiedFreq(freq).get(i)).zCoord != zCoord))
				temp.add(getStoragesOnSpecifiedFreq(freq).get(i));
		}
		if(temp.size() >= 1)
		{
			return true;
		}
		return false;
	}
	
	public Packet sendGuiData()
	{
		PG_PacketGuiSyncSend thePacket = new PG_PacketGuiSyncSend(xCoord, yCoord, zCoord);
		
		thePacket.integers = new int[3];
		thePacket.integers[0] = freq;
		thePacket.integers[1] = outputTier;
		thePacket.integers[2] = currentBehavior;
		
		thePacket.strings = new String[1];
		thePacket.strings[0] = owner;
		
		return thePacket.getPacket();
	}
	
	public Packet requestGuiData()
	{
		PG_PacketGuiSyncRequest thePacket = new PG_PacketGuiSyncRequest(xCoord, yCoord, zCoord);
		
		thePacket.flagFromFreq = new boolean[energyOnFreq.length];
		
		for(int i = 0; i < flagFromFreq.length; i++)
		{
			thePacket.flagFromFreq[i] = isThereSourceOn(i);
		}
		
		thePacket.source = sourceOfStorage;
		thePacket.size = getSharingStorages().size();
		thePacket.energy = energy;
		
		return thePacket.getPacket();
	}
	
	private boolean discharging; //0
	private boolean charging; //1
	private boolean resetted; //2
	protected boolean ownerIsSet; //3
	public boolean sourceOfStorage; //4
	public boolean addedToEnergyNet; //5
	
	public boolean[] flagFromFreq; //0
	
	public Container emptyContainer; //0
	
	private int counts = 0; //0
	protected int hPosForIcon = 0; //1
	public int freq; //2
	public int prevFreq; //3
	public int energy; //4
	public int maxStorage; //5
	public int output; //6
	public int tier; //7
	public int outputTier; //7
	public int currentBehavior; //8
	public int size; //9
	
	protected int[] inputForItemToCharge; //0
	protected int[] inputForItemToDischarge; //1
	public static int[] energyOnFreq = new int[100000]; //2
	public static int[] aOutput = {32, 128, 512}; //3
	
	public String owner; //0
	public String currentBehaviorName = ""; //1
	
	public ItemStack[] inventory; //0
	
	public static List<PG_TileEntityWirelessStorage> storageList = new LinkedList<PG_TileEntityWirelessStorage>(); //0
	
}
